package dev.pfaff.altargui.glfw;

import dev.pfaff.dyl4j.BundledLibraryLinker;
import dev.pfaff.glass.Arch;
import dev.pfaff.glass.Host;
import dev.pfaff.glass.OS;
import dev.pfaff.glass.glfw.GLFW;
import dev.pfaff.glass.opengl.GL;
import dev.pfaff.glass.opengl.GLSymbolLookup;
import dev.pfaff.log4truth.Logger;
import dev.pfaff.unfettered.Unfettered;

import java.io.IOException;
import java.io.InputStream;
import java.lang.foreign.Arena;
import java.lang.foreign.SymbolLookup;

import static java.util.FormatProcessor.FMT;

public final class GLGlobals {
	public static final GL GL;
	public static final GLFW GLFW = new GLFW(createGlfwLookup());

	private static SymbolLookup createGlfwLookup() {
		var libPath = System.getenv("GLFW_LIB_PATH");
		if (libPath != null) {
			// Path overload will canonicalize the path, which can fail on macOS with system libraries.
			//noinspection StringOperationCanBeSimplified
			return SymbolLookup.libraryLookup(libPath.toString(), Arena.ofAuto());
		}
		String libResourceName;
		switch (OS.HOST) {
			// TODO: search system library path
//			case OS.Linux _ -> {
//				// libglfw.so.3
//				return SymbolLookup.loaderLookup();
//			}
			case OS.Linux _ when Arch.HOST instanceof Arch.amd64 ->
					libResourceName = "/natives/libglfw-linux-amd64.so";
			case OS.macOS _ when Arch.HOST instanceof Arch.amd64 ->
					libResourceName = "/natives/libglfw.3.4-macos-amd64.dylib";
			case OS.macOS _ when Arch.HOST instanceof Arch.aarch64 ->
					libResourceName = "/natives/libglfw.3.4-macos-arm64.dylib";
			case OS.Windows _ when Arch.HOST instanceof Arch.amd64 ->
					libResourceName = "/natives/glfw3.4-windows-amd64-vc2022.dll";
			default -> throw Host.makeUnsupportedOSArchException();
		}
		InputStream inputStream = GLGlobals.class.getResourceAsStream(libResourceName);
		assert inputStream != null;
		try {
			return BundledLibraryLinker.createLookup(inputStream, Arena.ofAuto());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	static {
		try {
			var ALL_UNNAMED_MODULE = (Module) Unfettered.getTrustedLookup()
					.findStaticGetter(Module.class, "ALL_UNNAMED_MODULE", Module.class)
					.invokeExact();
			var VH = Unfettered.getTrustedLookup().findVarHandle(Module.class, "enableNativeAccess", boolean.class);
//			VH.compareAndSet(GLGlobals.class.getModule(), false, true);
			VH.compareAndSet(ALL_UNNAMED_MODULE, false, true);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}

		long start = System.nanoTime();

		GL = new GL(GLSymbolLookup.symbolLookupForHost());
		long elapsed = System.nanoTime() - start;
		Logger.log(() -> FMT."Initialized GL bindings in %.2f\{elapsed / 1_000_000.0} ms");
	}

	public static void forceInit() {
	}
}
