package dev.pfaff.altargui.glfw.ring;

import dev.pfaff.log4truth.format.Format;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;

public final class Stats {
	private final ObjectArrayList<Timing> timings = ObjectArrayList.wrap(Timing.EMPTY_TIMINGS_ARRAY);

	public Stats() {}

	public Stats timing(String name, LongRingBuffer timings) {
		this.timings.add(new Timing(name, timings));
		return this;
	}

	public String makeReport() {
		StringBuilder sb = new StringBuilder();

		sb.append("--- Stats ---\n");
		int padding = 0;
		for (var timing : timings) {
			padding = Math.max(padding, timing.name.length());
		}

		boolean notFirst = false;
		for (var timing : timings) {
			if (notFirst) sb.append('\n');
			notFirst = true;
			sb.append(timing.name)
					.append(" ".repeat(padding - timing.name.length()))
					.append(": ");
			Format.FIXED_DECIMAL_2.append(sb, timing.timings.avg());
			sb.append(" ns");
		}

		return sb.toString();
	}

	private record Timing(String name, LongRingBuffer timings) {
		public static final Timing[] EMPTY_TIMINGS_ARRAY = new Timing[0];
	}
}
